
p_ref = 2*10^(-5);     % reference acoustic pressure for Decibel scale;


%% Frequency range of study 

ka = logspace(log10(0.000001),log10(100),200001);       % Normalized frequency (by external radius of shell)

N = 250;                                         % Truncation degree of infinite series;

n_index = (0:N).';

lambda_n = n_index.*(n_index+1).*ka./ka;

%% Exterior fluid properties

rho = 1.21;              % fluid density (kg/m^3)
c = 343;                % speed of sound in fluid (m/s);



%% Spherical shell properties

a = 0.5;                % External radius of spherical shell (m)
E = 190*10^9;%          % Young`s modulus of shell material (Pa)
v = 0.27;              % Poisson`s ratio of shell material
rho_s = 7700;          % Density of shell material(kg/m^3)
p = 3;                 % Spherical shell thickness as percentage of external radius (%)
h = a*p/100;           % Spherical shell thickness (m)


beta_2 = h^2/(12*a^2);

cp_2 = E/(rho_s*(1-v^2));

omega = (c/sqrt(cp_2))*ka.*(n_index+1)./(n_index+1);


%% Spherical Bessel functions 

bessel_constant_h = zeros(N+1,numel(ka));  
bessel_constant_h_prime = zeros(N+1,numel(ka));
bessel_constant_j = zeros(N+1,numel(ka));  
bessel_constant_j_prime = zeros(N+1,numel(ka));
% bessel_constant_y = zeros(N+1,numel(ka));  
% bessel_constant_y_prime = zeros(N+1,numel(ka));    
    
   
for n = 0:N

    bessel_constant_h(n+1,:) = radial_functions('h2',n,ka.');  
    bessel_constant_h_prime(n+1,:) = radial_functions_prime('h2',n,ka.');
    bessel_constant_j(n+1,:) = radial_functions('j',n,ka.');  
    bessel_constant_j_prime(n+1,:) = radial_functions_prime('j',n,ka.');
    
%     bessel_constant_y(n+1,:) = radial_functions('y',n,ka.');  
%     bessel_constant_y_prime(n+1,:) = radial_functions_prime('y',n,ka.');
%     bessel_constant_h(n+1,:) = bessel_constant_j(n+1,:)-1i*bessel_constant_y(n+1,:);  
%     bessel_constant_h_prime(n+1,:) = bessel_constant_j_prime(n+1,:)-1i*bessel_constant_y_prime(n+1,:);
    
end    


bessel_constant_j(isnan(bessel_constant_j) == 1) = 0;  
bessel_constant_j_prime(isnan(bessel_constant_j_prime) == 1) = 0; 

bessel_constant_h(isnan(bessel_constant_h) == 1) = 0;
bessel_constant_h_prime(isnan(bessel_constant_h_prime) == 1) = 0;

% bessel_constant_y(isnan(bessel_constant_y) == 1) = 0;  
% bessel_constant_y_prime(isnan(bessel_constant_y_prime) == 1) = 0;


%% Impednace of external shell surface

Z_mech_n = 1i*rho_s*sqrt(cp_2)*(h/a)*(omega.^4-(1+3*v+lambda_n-beta_2*(1-v-lambda_n.^2-v*lambda_n)).*omega.^2+(beta_2*(lambda_n.^3-4*lambda_n.^2+lambda_n*(5-v^2)-2*(1-v^2))+(lambda_n-2)*(1-v^2)))./(omega.^3-omega.*(v+lambda_n-1)*(1+beta_2));

Z_l_n = Z_mech_n - 1i*rho*c*(bessel_constant_h./bessel_constant_h_prime);
%Z_l_n = 10^300;

%% Define frequency regime to be analysed 


the_ratio = (bessel_constant_j+1i*Z_mech_n/(rho*c).*bessel_constant_j_prime)./(bessel_constant_h+1i*Z_mech_n/(rho*c).*bessel_constant_h_prime);


var = (4./(ka.^2)).*(2*n_index+1).*abs(the_ratio).^2;

if max(max(isnan(var))) == 1
                    
    var(isnan(var) == 1) = 0;
     
end


scattered_power = sum(var,1);  
    




the_ratio2 = bessel_constant_j_prime./bessel_constant_h_prime;

var2 = (4./(ka.^2)).*(2*n_index+1).*abs(the_ratio2).^2;

if max(max(isnan(var2))) == 1
                    
    var2(isnan(var2) == 1) = 0;
     
end

     
scattered_power2 = sum(var2,1);  
    
   


%scattered_power_norm = scattered_power/(pi*a^2/(2*rho*c));



% figure();
% 
% %%%
% plot(ka',Z_scattered_power_dB,'-g','LineWidth',1.5);
% 
% xlabel(gca,'ka (log scale)','Fontsize',10,'Fontweight','b');
% ylabel(gca,'Scattered acoustic power, dB (ref = 10^{-12} W)','Fontsize',10,'Fontweight','b');
% set(gca,'Xscale','log');
% 
% title1 = title(gca,'Scattered power as a function of {\it ka} (Z = \rho_0\c_0 sphere)');
% title1.FontWeight = 'b';
% title1.FontSize = 14;
% 
% %axis(gca,[min(ka) max(ka) min(scattered_power_dB) ceil(max(scattered_power_dB)+2)]);
% grid(gca,'on');






% figure();
% 
% %%%
% plot(ka',Z_scattered_power/(2*pi*a^2/(rho*c)),'-g','LineWidth',1.5);
% 
% xlabel(gca,'ka (log scale)','Fontsize',10,'Fontweight','b');
% ylabel(gca,'(Scattered power) / (Incident power)','Fontsize',10,'Fontweight','b');
% set(gca,'Xscale','log');
% 
% title1 = title(gca,'Ratio of scattered power to incident power as a function of {\it ka} (Z = \rho_0\c_0 sphere)');
% title1.FontWeight = 'b';
% title1.FontSize = 14;
% 
% %axis(gca,[min(ka) max(ka) min(scattered_power_dB) ceil(max(scattered_power_dB)+2)]);
% grid(gca,'on');