%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  Written by Mihai Orita  (SPCG, ISVR, Southampton), in the period July 2017  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  Last updated on 21/07/2017, performed by Mihai Orita                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 


%% Physical constants (global)

rho = 1.2;             % air density for ideal conditions (kg/m^3)
c = 343;               % speed of sound in air under ideal conditions (m/s);
p_ref = 2*10^(-5);     % reference acoustic pressure for Decibel scale;

a = 0.3;               % Radius of rigid spherical scatterer (m)

Z = 0.01*rho*c;



%% Define frequency regime to be analysed 

ka = [0.5 5 50];
frequency = ka/a;  


                                                                      
%% Define spatial regions to be analysed

freq_max = max(ka/a);
spatial_resolution = (2*pi/freq_max)/10;

plane_angle = 0; 
limits = [-2 2 -2 2];
nr_a_values = ceil((limits(2)-limits(1))/spatial_resolution);
nr_b_values = ceil((limits(4)-limits(3))/spatial_resolution);


[x_grid, y_grid, z_grid] = PlanarSurface_GridCoordinates('rectangle',...
                           [plane_angle 0 0], limits, nr_a_values, nr_b_values);   % define points/grid in space (m) --> for evaluation;

% var_x = x_grid.';
% var_x_col = var_x(:,1);
% x_grid = var_x(var_x_col >= a*sqrt(2)/2,:).';
% 
% var_y = y_grid.';
% var_y_col = var_y(:,1);
% y_grid = var_y(var_y_col >= a*sqrt(2)/2,:).';
% 
% var_z = z_grid.';
% var_x_col = var_x(:,1);
% z_grid = var_x(var_x_col >= a*sqrt(2)/2,:).';


r_grid = sqrt(x_grid.^2+y_grid.^2+z_grid.^2);                           %
theta_grid = atan2(sqrt(x_grid.^2+y_grid.^2),z_grid);                   % separate spherical coordinates from spatial_info matrix into different arrays;
phi_grid = atan2(y_grid,x_grid);                                        %


var_no_object = abs(r_grid) < a;
x_grid(var_no_object) = NaN;                             %
y_grid(var_no_object) = NaN;                             %
z_grid(var_no_object) = NaN;                             % Give value of NaN to positions inside the volume of the scatterer to prevent plotting;
r_grid(var_no_object) = NaN;                             %
theta_grid(var_no_object) = NaN;                         %
phi_grid(var_no_object) = NaN;                           %


r = reshape(r_grid,nr_b_values*nr_a_values,1).';
theta = reshape(theta_grid,nr_b_values*nr_a_values,1).';
phi = reshape(phi_grid,nr_b_values*nr_a_values,1).';



%% 

var1 = bsxfun(@times,frequency.',ones(numel(frequency),numel(r)));     % initialise format and corresponding matrix for evaluating and, respectively, storing
bessel_N_input = bsxfun(@times,r,var1);                               % spherical Bessel functions --> frequency x position --> dictated by order of columns
                                                                      % in spatial_info;


%% Description of incident field (plane wave)

theta_i = pi/2;   % inclination angle of incidence from which the plane wave arrives;
phi_i = pi;       % azimuthal angle of incidence from which the plane wave arrives;

P0 = 1;           % magnitude of plane wave



%% Computations of pressure fields

% dB_value = 0.1/2;
% error_tolerance = p_ref*10^(dB_value/20);

error_tolerance = 0.000005;


% initialise current order that is processed in the main "while" loop;
N = 0;

% initialise matrix to store total pressure field of order N, format is frequency X position, where position is dictated by order of columns in spatial_info;
total_field_N = zeros(numel(frequency),numel(r));  

% initialise matrix to store total pressure field expanded as N-truncated series, format is frequency X position, where position is dictated by order of columns in spatial_info;
total_field = zeros(numel(frequency),numel(r)); 


flags_tf_dum = zeros(numel(frequency),numel(r));
total_field_N_max = zeros(numel(frequency),numel(r));
flags_tf = ones(numel(frequency),numel(r));


flag_while = 0 ; % binary switch that changes to "1" when 

% while flag_while == 0
% 
%     Y_i = spherical_harmonics(N,theta_i,phi_i);    % spherical harmonics of order n for angles of incidence of plane wave;
%     Y_spatial = spherical_harmonics(N,theta,phi);  % spherical harmonics of order n for discrete/grid points in space;
%         
%     first_sum_N = sum(bsxfun(@times,conj(Y_i),Y_spatial),1);         % first "infinite" sum in spherical harmonic expansion, matrix format 1x(number of positions in space);
%     
%     
%     %% Total pressure field
%     
%     bessel_descriptor_N1 = radial_functions('j',N,bessel_N_input);    % first type spherical Bessel functions of order n, matrix format (number of frequencies)x(number of positions in space);
%     bessel_descriptor_N2 = radial_functions('h2',N,bessel_N_input);   % first type spherical Bessel functions of order n, matrix format (number of frequencies)x(number of positions in space);
%     
%     var2 = ka;   % variable storing the values "ka";
%     
%     % variable storing the factor based of derivatives of Bessel functions, matrix format (number of frequencies)x1;
%     bessel_constant_j_N = radial_functions('j',N,var2.');  
%     bessel_constant_h_N = radial_functions('h2',N,var2.');  
%     bessel_constant_j_prime_N = radial_functions_prime('j',N,var2.');  
%     bessel_constant_h_prime_N = radial_functions_prime('h2',N,var2.'); 
%     
%     
%     alpha1 = 1i;
%     alpha2 = -Z./(rho*c);
%     
%     impedance_constant_N = ( bsxfun(@times,bessel_constant_j_N,alpha1) + bsxfun(@times,bessel_constant_j_prime_N,alpha2) ) ./ ( bsxfun(@times,bessel_constant_h_N,alpha1) + bsxfun(@times,bessel_constant_h_prime_N,alpha2) );
%     
%     
%     var3 = bsxfun(@times,impedance_constant_N,bessel_descriptor_N2);        % column-by-column multiplication;
%    
%     
%     % each term of order N in the second "infinite" sum of the spherical harmonic expansion;
%     total_field_N = P0*4*pi*(1i)^N*bsxfun(@times,first_sum_N,bessel_descriptor_N1-var3);
%     
%     
% %     if isnan(P0*4*pi*(-1i)^N*var5) == 1
% %         
% %         total_field_N = P0*4*pi*(-1i)^N*bsxfun(@times,first_sum_N,bessel_descriptor_N1); 
% %     
% %     else
% %         
% %         total_field_N = P0*4*pi*(-1i)^N*bsxfun(@times,first_sum_N,bessel_descriptor_N1-var5); 
% %         
% %     end
% 
%     flags_var3 = double(flags_tf_dum ~=2 & abs(total_field_N) <= error_tolerance);
%     flags_tf_dum = flags_tf_dum + flags_var3;
%     total_field_N_max = total_field_N_max + flags_var3*(N+1);
%     flags_tf_dum = changem(flags_tf_dum,2,1);
%     
%     flags_tf = double(abs(total_field_N) > error_tolerance);
%     
%     total_field_N(isnan(total_field_N) == 1) = 0;
%     
%     % scattered pressure field expanded as N-truncated series;
%     total_field = total_field + total_field_N.*flags_tf;
%     
% 
%     %%
% 
%     N = N+1;
%     
%     if sum(sum(flags_tf,1),2) == 0
% 
%         flag_while = 1;
%         
%     end
% 
% 
%     
%     
% end     

N = 150;
scattered_field = zeros(numel(frequency),numel(phi));
% incident_field = zeros(numel(frequency),numel(phi));

for n = 0:N

    Y_i = spherical_harmonics(n,theta_i,phi_i);    % spherical harmonics of order n for angles of incidence of plane wave;
    Y_spatial = spherical_harmonics(n,theta,phi);  % spherical harmonics of order n for discrete/grid points in space;
        
    first_sum_N = sum(bsxfun(@times,conj(Y_i),Y_spatial),1);         % first "infinite" sum in spherical harmonic expansion, matrix format 1x(number of positions in space);
    
    bessel_descriptor_N1 = radial_functions('j',n,bessel_N_input);    % first type spherical Bessel functions of order n, matrix format (number of frequencies)x(number of positions in space);
    bessel_descriptor_N2 = radial_functions('h2',n,bessel_N_input);   % first type spherical Bessel functions of order n, matrix format (number of frequencies)x(number of positions in space);
    
    var2 = ka;   % variable storing the values "ka";
    
    % variable storing the factor based of derivatives of Bessel functions, matrix format (number of frequencies)x1;
    bessel_constant_j_N = radial_functions('j',n,var2.');  
    bessel_constant_h_N = radial_functions('h2',n,var2.');  
    bessel_constant_j_prime_N = radial_functions_prime('j',n,var2.');  
    bessel_constant_h_prime_N = radial_functions_prime('h2',n,var2.'); 
    
    
    alpha1 = 1i;
    alpha2 = -Z./(rho*c);
    
    impedance_constant_N = ( bsxfun(@times,bessel_constant_j_N,alpha1) + bsxfun(@times,bessel_constant_j_prime_N,alpha2) ) ./ ( bsxfun(@times,bessel_constant_h_N,alpha1) + bsxfun(@times,bessel_constant_h_prime_N,alpha2) );
    
    
    var3 = bsxfun(@times,impedance_constant_N,bessel_descriptor_N2);        % column-by-column multiplication;
   

    var4 = -P0*4*pi*(1i)^n*bsxfun(@times,first_sum_N,var3);
    
    % each term of order N in the second "infinite" sum of the spherical harmonic expansion;

    var4(isnan(var4)) = 0;
        
    scattered_field = scattered_field + var4;
    
end



%%

x_grid = x_grid/a;
y_grid = y_grid/a;
z_grid = z_grid/a;

%save(['D:\Work\data\' 'rhoc_pw.mat'],'rho','c','a','P0','theta_i','phi_i','N','x_grid','y_grid','z_grid','total_field','scattered_field');

% 
% %%
figure();

% vector_colorbar = zeros(2,6);
% 
% for var_plot = 1:1:6
% 
% ka_string = num2str(ka(var_plot));
% 
% subplot(3,2,var_plot);

%%
% colormap = reshape(total_field(var_plot,:),nr_b_values,nr_a_values);
% 
% 
% %%%
% surface = surf(x_grid,y_grid,z_grid,20*log10(abs(colormap)./(2*10^(-5))));
% shading(gca,'flat');
% %alpha(surface1,1);
% 
% axis(gca,[limits/a -1 1]);
% axis equal;
% set(gca,'TickDir','out');
% set(gca,'XTick',-6:6);
% set(gca,'YTick',-6:6);
% 
% xlabel(gca,'x/a','Fontsize',8,'Fontweight','b');
% ylabel(gca,'y/a','Fontsize',8,'Fontweight','b');
% zlabel(gca,'z/a','Fontsize',8,'Fontweight','b');
% 
% 
% sPos = get(gca,'position');
% 
% colormap('jet');
% c_new = colorbar(gca);
% % vector_colorbar(:,var_plot-1) = caxis(gca);
% % colorbar 'off';
% c_new.Label.String = 'Total pressure field, dB (ref = 2*10^{-5} Pa)';
% c_new.Label.FontSize = 9;
% c_new.Location = 'southoutside';
% c_new.AxisLocation = 'out';
% 
% set(gca,'position',sPos);
% 
% 
% title1 = title(gca,['ka = ' ka_string]);
% title1.FontWeight = 'b';
% title1.FontSize = 16;
% 
% view(0,90);
% 
% end
% 
% % c_new = colorbar(gca);
% % 
% % var_c = [min(vector_colorbar(1,:)), max(vector_colorbar(2,:))];
% % caxis(var_c);
% % 
% % c_new.Label.String = 'Total pressure field, dB (ref = 2*10^{-5} Pa)';
% % c_new.Label.FontSize = 9;
% % c_new.Location = 'southoutside';
% % c_new.AxisLocation = 'out';
% 
% 
% 
% %%
% figure();
% 
% % vector_colorbar = zeros(2,6);
% 
% for var_plot = 1:1:6
% 
% ka_string = num2str(ka(var_plot));
% 
% subplot(3,2,var_plot);
% 
% %%
% colormap = reshape(scattered_field(var_plot,:),nr_b_values,nr_a_values);
% 
% 
% %%%
% surface = surf(x_grid,y_grid,z_grid,20*log10(abs(colormap)./(2*10^(-5))));
% shading(gca,'flat');
% %alpha(surface1,1);
% 
% axis(gca,[limits/a -1 1]);
% axis equal;
% set(gca,'TickDir','out');
% set(gca,'XTick',-6:6);
% set(gca,'YTick',-6:6);
% 
% xlabel(gca,'x/a','Fontsize',8,'Fontweight','b');
% ylabel(gca,'y/a','Fontsize',8,'Fontweight','b');
% zlabel(gca,'z/a','Fontsize',8,'Fontweight','b');
% 
% 
% sPos = get(gca,'position');
% 
% colormap('jet');
% c_new = colorbar(gca);
% % vector_colorbar(:,var_plot-1) = caxis(gca);
% % colorbar 'off';
% c_new.Label.String = 'Scattered pressure field, dB (ref = 2*10^{-5} Pa)';
% c_new.Label.FontSize = 9;
% c_new.Location = 'southoutside';
% c_new.AxisLocation = 'out';
% 
% set(gca,'position',sPos);
% 
% 
% title1 = title(gca,['ka = ' ka_string]);
% title1.FontWeight = 'b';
% title1.FontSize = 16;
% 
% view(0,90);
% 
% end
% 
% % c_new = colorbar(gca);
% % 
% % var_c = [min(vector_colorbar(1,:)), max(vector_colorbar(2,:))];
% % caxis(var_c);
% % 
% % c_new.Label.String = 'Scattered pressure field, dB (ref = 2*10^{-5} Pa)';
% % c_new.Label.FontSize = 9;
% % c_new.Location = 'southoutside';
% % c_new.AxisLocation = 'out';
% 
% 
% 
% %%
% figure();
% 
% % vector_colorbar = zeros(2,6);
% 
% for var_plot = 1:1:6
% 
% ka_string = num2str(ka(var_plot));
% 
% subplot(3,2,var_plot);
% 
% %%
% colormap = reshape(total_field(var_plot,:),nr_b_values,nr_a_values);
% 
% 
% %%%
% surface = surf(x_grid,y_grid,z_grid,real(colormap));
% shading(gca,'flat');
% %alpha(surface1,1);
% 
% axis(gca,[limits/a -1 1]);
% axis equal;
% set(gca,'TickDir','out');
% set(gca,'XTick',-6:6);
% set(gca,'YTick',-6:6);
% 
% xlabel(gca,'x/a','Fontsize',8,'Fontweight','b');
% ylabel(gca,'y/a','Fontsize',8,'Fontweight','b');
% zlabel(gca,'z/a','Fontsize',8,'Fontweight','b');
% 
% 
% sPos = get(gca,'position');
% 
% colormap('jet');
% c_new = colorbar(gca);
% % vector_colorbar(:,var_plot-1) = caxis(gca);
% % colorbar 'off';
% c_new.Label.String = 'Total pressure field, Pa';
% c_new.Label.FontSize = 9;
% c_new.Location = 'southoutside';
% c_new.AxisLocation = 'out';
% 
% set(gca,'position',sPos);
% 
% 
% title1 = title(gca,['ka = ' ka_string]);
% title1.FontWeight = 'b';
% title1.FontSize = 16;
% 
% view(0,90);
% 
% end
% 
% % c_new = colorbar(gca);
% % 
% % var_c = [min(vector_colorbar(1,:)), max(vector_colorbar(2,:))];
% % caxis(var_c);
% % 
% % c_new.Label.String = 'Total pressure field, Pa';
% % c_new.Label.FontSize = 9;
% % c_new.Location = 'southoutside';
% % c_new.AxisLocation = 'out';
% 
% 
% %%
% figure();
% 
% % vector_colorbar = zeros(2,6);
% 
% for var_plot = 1:1:6
% 
% ka_string = num2str(ka(var_plot));
% 
% subplot(3,2,var_plot);
% 
% %%
% colormap = reshape(scattered_field(var_plot,:),nr_b_values,nr_a_values);
% 
% 
% %%%
% surface = surf(x_grid,y_grid,z_grid,real(colormap));
% shading(gca,'flat');
% %alpha(surface1,1);
% 
% axis(gca,[limits/a -1 1]);
% axis equal;
% set(gca,'TickDir','out');
% set(gca,'XTick',-6:6);
% set(gca,'YTick',-6:6);
% 
% xlabel(gca,'x/a','Fontsize',8,'Fontweight','b');
% ylabel(gca,'y/a','Fontsize',8,'Fontweight','b');
% zlabel(gca,'z/a','Fontsize',8,'Fontweight','b');
% 
% 
% sPos = get(gca,'position');
% 
% colormap('jet');
% c_new = colorbar;
% % vector_colorbar(:,var_plot-1) = caxis(gca);
% c_new.Label.String = 'Scattered pressure field, Pa';
% c_new.Label.FontSize = 9;
% c_new.Location = 'southoutside';
% c_new.AxisLocation = 'out';
% 
% set(gca,'position',sPos);
% 
% 
% title1 = title(gca,['ka = ' ka_string]);
% title1.FontWeight = 'b';
% title1.FontSize = 16;
% 
% view(0,90);
% 
% end
% 
% % colorbar(gca);
% % var_c = [min(vector_colorbar(1,:)), max(vector_colorbar(2,:))];
% % caxis(var_c);
% % 
% % c_new.Label.String = 'Scattered pressure field, Pa';
% % c_new.Label.FontSize = 9;
% % c_new.Location = 'southoutside';
% % c_new.AxisLocation = 'out';