function [CSHSC_vs_ka, RSHSC_vs_ka, NSHSA_vs_ka] = acous_scattering_SHSC( id, Z_n, scaler, r_l, theta_l, phi_l, a, rho, c, freq)
%UNTITLED2 Summary of this function goes here
%
%   id = 'pw', 'pf'  or  'pm'
%   freq  =   1 x numel(freq)
%   Z_n   =   modal (N) x freq (numel freq)
%   r_l , theta_l , phi_l  = 1 x L (numel r_l)

%% Predefine

ka = a*2*pi*freq/c;

Zeta_n = Z_n/(rho*c);

N = numel(sum(Z_n,2))-1;

L = numel(r_l);

CSHSC_vs_ka = zeros((N+1)^2,numel(ka));
RSHSC_vs_ka = zeros((N+1)^2,numel(ka));
NSHSA_vs_ka = zeros(N+1,numel(ka));



%%

for n = 0:N

    
    % spherical harmonic functions

    Y_n_m = spherical_harmonics(n,theta_l,phi_l);
    
    
    % plane- wave scattering
    
    if strcmp(id,'pw') == 1                                                   % Choice is spherical Bessel function of the first kind;
    
        
        bessel_constant_j = radial_functions('j',n,ka);  
        bessel_constant_h = radial_functions('h2',n,ka);  
        bessel_constant_j_prime = radial_functions_prime('j',n,ka);  
        bessel_constant_h_prime = radial_functions_prime('h2',n,ka);  
        
        bessel_constant_j(isnan(bessel_constant_j) == 1) = 0;  
        bessel_constant_j_prime(isnan(bessel_constant_j_prime) == 1) = 0; 
        bessel_constant_h(isnan(bessel_constant_h) == 1) = 0;
        bessel_constant_h_prime(isnan(bessel_constant_h_prime) == 1) = 0;
        
        
        NSHSA_vs_ka(n+1,:) = (bessel_constant_j+1i*Zeta_n(n+1,:).*bessel_constant_j_prime)./(bessel_constant_h+1i*Zeta_n(n+1,:).*bessel_constant_h_prime);                                                                                                                 

        CSHSC_vs_ka((n^2+1):(n+1)^2,:) = scaler*4*pi*1i^(n+2)*NSHSA_vs_ka(n+1,:).*conj(Y_n_m);

        
        if n == 0
        
            RSHSC_vs_ka(1,:) = CSHSC_vs_ka(1,:);
        
        else
        
            RSHSC_vs_ka(n^2+1:n^2+2*n+1,:) =  (sqrt(2)/2)*[(bsxfun(@times,((-1).^[n:-1:1]).',-CSHSC_vs_ka(n^2+1:n^2+n,:)) + CSHSC_vs_ka(n^2+n+2:n^2+2*n+1,:))*1i; (2/sqrt(2))*CSHSC_vs_ka(n^2+n+1,:); ...
                                                     bsxfun(@times,((-1).^[1:1:n]).',CSHSC_vs_ka(n^2+1:n^2+n,:)) + CSHSC_vs_ka(n^2+n+2:n^2+2*n+1,:)]; 
          
        end
        
        
    end
    
    
    % point-force on surface
    
    if strcmp(id,'pf') == 1                                                   % Choice is spherical Bessel function of the first kind;
    
        
        bessel_constant_h = radial_functions('h2',n,ka);    
        bessel_constant_h_prime = radial_functions_prime('h2',n,ka);  
        
        bessel_constant_h(isnan(bessel_constant_h) == 1) = 0;
        bessel_constant_h_prime(isnan(bessel_constant_h_prime) == 1) = 0;
        
        
        NSHSA_vs_ka(n+1,:) = (bessel_constant_h_prime./(bessel_constant_h+1i*Zeta_n(n+1,:).*bessel_constant_h_prime)).*conj(Y_n_m(n+1));                                                                                                                 

        CSHSC_vs_ka((n^2+1):(n+1)^2,:) = scaler*(1/a^2)*NSHSA_vs_ka(n+1,:).*conj(Y_n_m);

        
        if n == 0
        
            RSHSC_vs_ka(1,:) = CSHSC_vs_ka(1,:);
        
        else
        
            RSHSC_vs_ka(n^2+1:n^2+2*n+1,:) =  (sqrt(2)/2)*[(bsxfun(@times,((-1).^[n:-1:1]).',-CSHSC_vs_ka(n^2+1:n^2+n,:)) + CSHSC_vs_ka(n^2+n+2:n^2+2*n+1,:))*1i; (2/sqrt(2))*CSHSC_vs_ka(n^2+n+1,:); ...
                                                     bsxfun(@times,((-1).^[1:1:n]).',CSHSC_vs_ka(n^2+1:n^2+n,:)) + CSHSC_vs_ka(n^2+n+2:n^2+2*n+1,:)]; 
          
        end
        
        
    end
    
    
    % point-monopole on surface
    
    if strcmp(id,'pm') == 1                                                   % Choice is spherical Bessel function of the first kind;
    
         
        bessel_constant_h = radial_functions('h2',n,ka);   
        bessel_constant_h_prime = radial_functions_prime('h2',n,ka);  
        
        bessel_constant_h(isnan(bessel_constant_h) == 1) = 0;
        bessel_constant_h_prime(isnan(bessel_constant_h_prime) == 1) = 0;
        
        
        NSHSA_vs_ka(n+1,:) = Zeta_n(n+1,:)./(bessel_constant_h+1i*Zeta_n(n+1).*bessel_constant_h_prime);                                                                                                                 

        CSHSC_vs_ka((n^2+1):(n+1)^2,:) = scaler*rho*c*(1/a^2)*NSHSA_vs_ka(n+1,:).*conj(Y_n_m);

        
        if n == 0
        
            RSHSC_vs_ka(1,:) = CSHSC_vs_ka(1,:);
        
        else
        
            RSHSC_vs_ka(n^2+1:n^2+2*n+1,:) =  (sqrt(2)/2)*[(bsxfun(@times,((-1).^[n:-1:1]).',-CSHSC_vs_ka(n^2+1:n^2+n,:)) + CSHSC_vs_ka(n^2+n+2:n^2+2*n+1,:))*1i; (2/sqrt(2))*CSHSC_vs_ka(n^2+n+1,:); ...
                                                     bsxfun(@times,((-1).^[1:1:n]).',CSHSC_vs_ka(n^2+1:n^2+n,:)) + CSHSC_vs_ka(n^2+n+2:n^2+2*n+1,:)]; 
          
        end
        
        
    end
    
    
    % plane- wave free-field (incoming)
    
    if strcmp(id,'pi') == 1                                                   % Choice is spherical Bessel function of the first kind;
    
        
        bessel_constant_j = radial_functions('j',n,ka);
        
        bessel_constant_j(isnan(bessel_constant_j) == 1) = 0;  
        
        %                                                                                   

        CSHSC_vs_ka((n^2+1):(n+1)^2,:) = scaler*4*pi*1i^(n)*bessel_constant_j.*conj(Y_n_m);

        
        if n == 0
        
            RSHSC_vs_ka(1,:) = CSHSC_vs_ka(1,:);
        
        else
        
            RSHSC_vs_ka(n^2+1:n^2+2*n+1,:) =  (sqrt(2)/2)*[(bsxfun(@times,((-1).^[n:-1:1]).',-CSHSC_vs_ka(n^2+1:n^2+n,:)) + CSHSC_vs_ka(n^2+n+2:n^2+2*n+1,:))*1i; (2/sqrt(2))*CSHSC_vs_ka(n^2+n+1,:); ...
                                                     bsxfun(@times,((-1).^[1:1:n]).',CSHSC_vs_ka(n^2+1:n^2+n,:)) + CSHSC_vs_ka(n^2+n+2:n^2+2*n+1,:)]; 
          
        end
        
        
    end
    
    
    
    NSHSA_vs_ka(isnan(NSHSA_vs_ka) == 1) = 0;
    CSHSC_vs_ka(isnan(CSHSC_vs_ka) == 1) = 0;
    RSHSC_vs_ka(isnan(RSHSC_vs_ka) == 1) = 0;
    

end