
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  Written by Mihai Orita  (SPCG, ISVR, Southampton), in the period July 2017  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  Last updated on 21/07/2017, performed by Mihai Orita                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

for many = 1:200

    [sphere]=geometry_2(0.3, many);
    r = sqrt((sphere(:,1)*cos(-pi/2)+sphere(:,3)*sin(-pi/2)).^2+(sphere(:,2)).^2+(-sphere(:,1)*sin(-pi/2)+sphere(:,3)*cos(-pi/2)).^2);
    secondary_sources=[zeros(size(sphere,1),1) r acos((-sphere(:,1)*sin(-pi/2)+sphere(:,3)*cos(-pi/2))./r) atan2(sphere(:,2),sphere(:,1)*cos(-pi/2)+sphere(:,3)*sin(-pi/2))];



    %% Physical constants (global)


    % Properties of propagation medium

    rho = 1.2;             % air density for ideal conditions [g/m^3]
    c = 343;               % speed of sound in air under ideal conditions [m/s]

    % Properties of obstacle in free-field

    a = 0.3;               % Radius of rigid spherical scatterer [m]
    %Z = rho*c;             % Uniform, locall-reacting impedance on the surface of obstacle [Pa*s/m^3]


    % Auxiliary constants for calculations 

    p_ref = 2*10^(-5);     % reference acoustic pressure for Decibel scale [Pa]
    W_ref = 10^(-12);      % reference acoustic power for Decibel scale [W]



    %% Define frequency axis and truncation orders

    ka = logspace(log10(0.1),log10(100),501).';  % dimensionless wavenumber []

    %N = ceil(4*ka);                                   % truncation order for SHS describing total sound power after control
    N = 400*ones(size(ka));

    zeta_norm = ones(1,numel(ka))*100/101;

    Pi = 1;

    %% Define primary sources (disturbances)

    % row format: | multipole order | r | theta | phi |

    primary_sources = [Inf Inf pi pi/2];

    nr_p_s = size(primary_sources,1);



    %% Define secondary sources

    % row format: | multipole order | r | theta | phi |

    %secondary_sources = [NaN 0.3 pi/2 0; NaN 0.3 pi/2 pi; NaN 0.3 3*pi/4 pi];

    nr_s_s = size(secondary_sources,1);



    %% Pre

    flag_type = 'real';

    conglomerate = zeros(size(primary_sources,1)+size(secondary_sources,1),4);
    conglomerate([1:nr_p_s (nr_p_s+1):(nr_p_s+nr_s_s)] ,:) = [primary_sources; secondary_sources];

    [cong_sort_ord,i1_ord,i2_ord] = unique(conglomerate(:,1));
    [cong_sort_r,i1_r,i2_r] = unique(conglomerate(:,2));
    [cong_sort_theta,i1_theta,i2_theta] = unique(abs(cos(conglomerate(:,3))));
    [cong_sort_phi,i1_phi,i2_phi] = unique(conglomerate(:,4));

    nr_sort_r = numel(cong_sort_r);
    nr_sort_theta = numel(cong_sort_theta);

    var = cos(conglomerate(:,3))<0;



    %A_sort_theta_pos = A_sort_theta(A_sort_theta>=0);


    %p_s_SH_factors = conglomerate();


    %s_s_SH_factors = conglomerate();


    % a_legendre_f  = zeros((N+1)*(N+2)/2,numel(A_sort_theta));
    % exp_m = zeros((N+1)*(N+2)/2,numel(A_sort_phi));
    %  
    % bessel_j_ka = zeros(1,numel(ka));
    % bessel_h2_ka = zeros(1,numel(ka));
    % bessel_prime_j_ka = zeros(1,numel(ka));
    % bessel_prime_h2_ka = zeros(1,numel(ka));

    q_opt = zeros(nr_s_s,numel(ka))*NaN;
    pow_cont = zeros(size(ka))*NaN;


    for index = 1:numel(ka)



        primary = zeros((N(index)+1)^2,nr_p_s);
        secondary = zeros((N(index)+1)^2, nr_s_s);
        output = zeros((N(index)+1)^2,nr_p_s + nr_s_s);

        n_m_index_arrangement = zeros((N(index)+1)^2,2);

        a_legendre_f_N  = zeros((N(index)+1)^2,nr_sort_theta);

        exp_m_N = zeros((N(index)+1)^2,numel(cong_sort_phi));

        bessel_j_ka = zeros(N(index)+1,1);
        bessel_h2_ka = zeros(N(index)+1,1);
        bessel_prime_j_ka = zeros(N(index)+1,1);
        bessel_prime_h2_ka = zeros(N(index)+1,1);

        %bessel_j_kr = zeros(N(index)+1,nr_sort_r);
        %bessel_h2_kr = zeros(N(index)+1,nr_sort_r);


        extra_consts1 = zeros((N(index)+1)^2,1);
        %extra_consts2 = zeros(N(index)+1,1);


        for n = 0:N(index)

            n_m_index_arrangement(n^2+1:(n+1)^2,1) = (n+1)*ones(2*n+1,1);
            n_m_index_arrangement(n^2+1:(n+1)^2,2) = 0:2*n;

            % build theta dependent factors of positive m
            a_legendre_f_N(n^2+n+1:(n+1)^2,nr_sort_theta-nr_sort_theta+1:end) = ...
                bsxfun(@times,(-1).^(0:1:n).',sqrt(1/(2*pi)).*legendre(n,cong_sort_theta,'norm'));

            % build theta dependent factors of negative m from those of positive m . n=0 index calls upon an empty matrix to get a value,
            % thus 'a_legendre_f_N' remains unchanged.
            a_legendre_f_N(n^2+1:(n+1)^2-n-1,nr_sort_theta-nr_sort_theta+1:end) = ...
                bsxfun(@times,(-1).^(1:1:n).',a_legendre_f_N(n^2+n+2:(n+1)^2,nr_sort_theta-nr_sort_theta+1:end));

            %%.*(gamma(repmat(1:1:n,numel(A_sort_theta_pos),1)-(n-1)*ones(numel(A_sort_theta_pos,n)))./gamma(repmat(1:1:n,numel(A_sort_theta_pos),1))+(n+1)*ones(numel(A_sort_theta_pos),n))...


            % build phi dependent factors
            exp_m_N(n^2+1:(n+1)^2,:) = exp(bsxfun(@times,(-n:1:n).',1i*cong_sort_phi.'));


            bessel_j_ka(n+1,1) = sqrt(pi./(2*ka(index))).*besselj(n+0.5,ka(index));
            bessel_h2_ka(n+1,1) = sqrt(pi./(2*ka(index))).*conj(besselh(n+0.5,ka(index)));

            bessel_prime_j_ka(n+1,1) = sqrt(pi/(2*ka(index)))*besselj(n-1+0.5,ka(index))-(2*n+1)*bessel_j_ka(n+1,1)/ka(index);
            bessel_prime_h2_ka(n+1,1) = sqrt(pi/(2*ka(index)))*conj(besselh(n-1+0.5,ka(index)))-(2*n+1)*bessel_h2_ka(n+1,1)/ka(index);   

            %bessel_j_kr(n+1,:) = sqrt(pi./(2*cong_sort_r.'*ka(index)/a)).*besselj(n+0.5,cong_sort_r.'*ka(index)/a);
            %bessel_h2_kr(n+1,:) = sqrt(pi./(2*cong_sort_r.'*ka(index)/a)).*besselh(n+0.5,cong_sort_r.'*ka(index)/a);


        end


        %SH_coeff_pow_contrib_p_s = abs(repmat(bessel_ratio.',numel(conglomerate(1:size(primary_sources,1),3)),1).* SH_coeff_angular(1:size(primary_sources,1),:)).^2;


        %SH_coeff_pow_contrib_s_s = abs(repmat((bessel_ratio).',numel(conglomerate(1:size(secondary_sources,1),3)),1).* SH_coeff_angular(size(primary_sources,1)+1:end,:)).^2;


        extra_consts1(:) = Pi*4*pi*(1i).^(n_m_index_arrangement(:,1)+2-1);
        %extra_consts2(n+1,1) = rh0*c/a^2;


        output(:) = a_legendre_f_N(:,i2_theta).*exp_m_N(:,i2_phi);

        % build theta dependent factors of negative argument from those of positive argument
        output(:,var) = conj(bsxfun(@times,(-1).^n_m_index_arrangement(:,2),output(:,var)));


        %
        primary(:) = bsxfun(@times,extra_consts1.*((1-zeta_norm(index))*bessel_j_ka(n_m_index_arrangement(:,1))+1i*zeta_norm(index).*bessel_prime_j_ka(n_m_index_arrangement(:,1)))./...
                          ((1-zeta_norm(index))*bessel_h2_ka(n_m_index_arrangement(:,1))+1i*zeta_norm(index).*bessel_prime_h2_ka(n_m_index_arrangement(:,1))),output(:,1:nr_p_s));



        secondary(:) = bsxfun(@times,(rho*c/a^2)*(zeta_norm(index))./...
                          ((1-zeta_norm(index))*bessel_h2_ka(n_m_index_arrangement(:,1))+1i*zeta_norm(index).*bessel_prime_h2_ka(n_m_index_arrangement(:,1))),output(:,nr_p_s+1:nr_p_s+nr_s_s));

    %     secondary(:) = bsxfun(@times,rho*c*(ka(index)/a)^2*(bessel_j_ka(n_m_index_arrangement(:,1))-((1-zeta_norm(index))*bessel_j_ka(n_m_index_arrangement(:,1))+1i*zeta_norm(index).*bessel_prime_j_ka(n_m_index_arrangement(:,1)))./...
    %                       ((1-zeta_norm(index))*bessel_h2_ka(n_m_index_arrangement(:,1))+1i*zeta_norm(index).*bessel_prime_h2_ka(n_m_index_arrangement(:,1))).*bessel_h2_ka(n_m_index_arrangement(:,1))),...
    %                       output(:,nr_p_s+1:nr_p_s+nr_s_s));


        % get rid of NaN values that can occur when coefficient becomes too small in magnitude.        
        primary(isnan(primary) == 1) = 0;      
        secondary(isnan(secondary) == 1) = 0;


    %      if strcmp(flag_type,'real') == 1
    %          
    %          primary((-n_m_index_arrangement(:,1)+1+n_m_index_arrangement(:,2))<0,:) = 0.5*1i*sqrt(2)*(primary((-n_m_index_arrangement(:,1)+1+n_m_index_arrangement(:,2))>0,:)-...
    %                                                                                     primary((-n_m_index_arrangement(:,1)+1+n_m_index_arrangement(:,2))<0,:));
    %          primary((-n_m_index_arrangement(:,1)+1+n_m_index_arrangement(:,2))>0,:) = 0.5*sqrt(2)*(primary((-n_m_index_arrangement(:,1)+1+n_m_index_arrangement(:,2))>0,:)+...
    %                                                                                     primary((-n_m_index_arrangement(:,1)+1+n_m_index_arrangement(:,2))<0,:));
    %          
    %                                                                                 
    %          secondary((-n_m_index_arrangement(:,1)+1+n_m_index_arrangement(:,2))<0,:) = 1i*0.5*sqrt(2)*(secondary((-n_m_index_arrangement(:,1)+1+n_m_index_arrangement(:,2))>0,:)-...
    %                                                                                     secondary((-n_m_index_arrangement(:,1)+1+n_m_index_arrangement(:,2))<0,:));
    %          secondary((-n_m_index_arrangement(:,1)+1+n_m_index_arrangement(:,2))>0,:) = 0.5*sqrt(2)*(secondary((-n_m_index_arrangement(:,1)+1+n_m_index_arrangement(:,2))>0,:)+...
    %                                                                                     secondary((-n_m_index_arrangement(:,1)+1+n_m_index_arrangement(:,2))<0,:));
    % 
    %      end


        % build spherical harmonics of something phi argument
        
        
        if abs(cond(secondary'*secondary,2)) > 10^6

            continue;

        end


        q_opt(:,index) = -(secondary'*secondary)\(secondary'*primary);


        pow_cont(index) = sum(abs(primary + sum(bsxfun(@times,q_opt(:,index).',secondary),2)).^2,1)/(2*rho*c*(ka(index)/a)^2);


    end


    %save(['C:\Downloads\' 'Z100_many' num2str(many+1) '.mat'],'many','q_opt','pow_cont');



end
