
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  Written by Mihai Orita  (SPCG, ISVR, Southampton), in the period July 2017  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  Last updated on 21/07/2017, performed by Mihai Orita                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 


%% Physical constants (global)

rho = 1.2;             % air density for ideal conditions (kg/m^3)
c = 343;               % speed of sound in air under ideal conditions (m/s);
p_ref = 2*10^(-5);     % reference acoustic pressure for Decibel scale;

a = 0.3;               % Radius of rigid spherical scatterer (m)

Z = rho*c;



%% Define frequency regime to be analysed 

ka = [0.5 1 3 6 10 20];
wavenumber = ka/a;  


                                                                      
%% Define spatial regions to be analysed

% freq_max = max(ka/a);
% spatial_resolution = (2*pi/freq_max)/6;

nr_phi_values = 10000;

phi = linspace(0,pi,nr_phi_values);             
theta = pi/2*ones(1,numel(phi));          
r = 1*ones(1,numel(phi));

% x = cos(phi).* sin(theta) .*r;
% y = sin(phi).* sin(theta) .*r;
% z = cos(theta) .*r;



%% 

var1 = bsxfun(@times,wavenumber.',ones(numel(wavenumber),numel(phi)));     % initialise format and corresponding matrix for evaluating and, respectively, storing

%Z = Z*ones(numel(wavenumber),numel(r));


                                                                      
%% Description of incident field (plane wave)

theta_i = pi/2;   % inclination angle of incidence from which the plane wave arrives;
phi_i = pi;       % azimuthal angle of incidence from which the plane wave arrives;

P0 = 1;           % magnitude of plane wave



%% Computations of pressure fields

% dB_value = 0.1/2;
% error_tolerance = p_ref*10^(dB_value/20);

error_tolerance = 0.000005;


% initialise current order that is processed in the main "while" loop;
N = 0;

% initialise matrix for storing incident plane wave pressure field of order N, matrix format is (number of frequencies)x(number of positions in space);    
norm_ffp_N = zeros(numel(wavenumber),numel(r)); 

% initialise matrix for storing incident plane wave pressure field expanded as N-truncated series, matrix format is (number of frequencies)x(number of positions in space);
norm_ffp = zeros(numel(wavenumber),numel(r)); 


flags_nffp_dum = zeros(numel(wavenumber),numel(r));
norm_ffp_N_max = zeros(numel(wavenumber),numel(r));
flags_norm_ffp = ones(numel(wavenumber),numel(r));


flag_while = 0 ; % binary switch that changes to "1" when 

while flag_while == 0

    Y_i = spherical_harmonics(N,theta_i,phi_i);    % spherical harmonics of order n for angles of incidence of plane wave;
    Y_spatial = spherical_harmonics(N,theta,phi);  % spherical harmonics of order n for discrete/grid points in space;
        
    first_sum_N = sum(bsxfun(@times,conj(Y_i),Y_spatial),1);         % first "infinite" sum in spherical harmonic expansion, matrix format 1x(number of positions in space);
    
    var2 = ka;   % variable storing the values "ka";
    
    % variable storing the factor based of derivatives of Bessel functions, matrix format (number of frequencies)x1;
    bessel_constant_j_N = radial_functions('j',N,var2.');  
    bessel_constant_h_N = radial_functions('h2',N,var2.');  
    bessel_constant_j_prime_N = radial_functions_prime('j',N,var2.');  
    bessel_constant_h_prime_N = radial_functions_prime('h2',N,var2.');
    
    alpha1 = 1i;
    alpha2 = -Z/(rho*c);
    
    impedance_constant_N = ( bsxfun(@times,bessel_constant_j_N,alpha1) + bsxfun(@times,bessel_constant_j_prime_N,alpha2) ) ./ ( bsxfun(@times,bessel_constant_h_N,alpha1) + bsxfun(@times,bessel_constant_h_prime_N,alpha2) ); 

    var3 = bsxfun(@times,bsxfun(@times,1./var2',impedance_constant_N),var1);
    
    % each term of order N in the second "infinite" sum of the spherical harmonic expansion;
    norm_ffp_N = - 4*pi*(1i)^(2*N+1)*bsxfun(@times,first_sum_N,var3);  
    

    flags_var1 = double(flags_nffp_dum ~=2 & abs(norm_ffp_N) <= error_tolerance);
    flags_nffp_dum = flags_nffp_dum + flags_var1;
    norm_ffp_N_max = norm_ffp_N_max + flags_var1*(N+1);
    flags_nffp_dum = changem(flags_nffp_dum,2,1);
    
    flags_norm_ffp = double(abs(norm_ffp_N) > error_tolerance);
    
    norm_ffp_N(isnan(norm_ffp_N) == 1) = 0;
    
    % incident pressure field expanded as N-truncated series;
    norm_ffp = norm_ffp + norm_ffp_N.*flags_norm_ffp;
    
    
    %%

    N = N+1;
    
    if sum(sum(flags_norm_ffp,1),2) == 0

        flag_while = 1;
        
    end
      
end

% x = cos(phi_grid).* sin(theta_grid) .*abs(n_ff_scattered_pressure_grid);
% y = sin(phi_grid).* sin(theta_grid) .*abs(n_ff_scattered_pressure_grid);
% z = cos(theta_grid) .*abs(n_ff_scattered_pressure_grid);

% x_plot = bsxfun(@times,cos(phi),abs(norm_ffp(2,:)));
% y_plot = bsxfun(@times,sin(phi),abs(norm_ffp(2,:)));

n = N;
n_ff_scattered_pressure = zeros(numel(wavenumber),numel(phi));

for N = 0:n

    Y_i = spherical_harmonics(N,theta_i,phi_i);    % spherical harmonics of order n for angles of incidence of plane wave;
    Y_spatial = spherical_harmonics(N,theta,phi);  % spherical harmonics of order n for discrete/grid points in space;
        
    first_sum_N = sum(bsxfun(@times,conj(Y_i),Y_spatial),1);         % first "infinite" sum in spherical harmonic expansion, matrix format 1x(number of positions in space);
    
    var2 = ka;   % variable storing the values "ka";
    
    % variable storing the factor based of derivatives of Bessel functions, matrix format (number of frequencies)x1;
    bessel_constant_j_N = radial_functions('j',N,var2.');  
    bessel_constant_h_N = radial_functions('h2',N,var2.');  
    bessel_constant_j_prime_N = radial_functions_prime('j',N,var2.');  
    bessel_constant_h_prime_N = radial_functions_prime('h2',N,var2.');
    
    alpha1 = 1i;
    alpha2 = -Z/(rho*c);
    
    impedance_constant_N = ( bsxfun(@times,bessel_constant_j_N,alpha1) + bsxfun(@times,bessel_constant_j_prime_N,alpha2) ) ./ ( bsxfun(@times,bessel_constant_h_N,alpha1) + bsxfun(@times,bessel_constant_h_prime_N,alpha2) );  

    var3 = -4*pi*(1i)^(2*N+1)*bsxfun(@times,first_sum_N,bsxfun(@times,1./var2.',impedance_constant_N));
    
    % each term of order N in the second "infinite" sum of the spherical harmonic expansion;
    n_ff_scattered_pressure = n_ff_scattered_pressure + var3;

end

% x = cos(phi).* sin(theta) .*abs(norm_ffp(2,:));
% y = sin(phi).* sin(theta) .*abs(norm_ffp(2,:));
% z = cos(theta) .*abs(norm_ffp(2,:));


figure();

for var_plot = 1:1:6

ka_string = num2str(ka(var_plot));

subplot(3,2,var_plot);

%%%
polarplot(phi,abs(n_ff_scattered_pressure(var_plot,:)),'-b','LineWidth',1);

pax = gca;
pax.ThetaAxisUnits = 'radians';

help = [0.5 1 2.5 4 2.5 2.5];
rlim([0 help(var_plot)]);
% pax.RTick = [0 0.05 0.1 0.15 0.2];

% xlabel('x','Fontsize',10,'Fontweight','b');
% ylabel('y','Fontsize',10,'Fontweight','b');

title1 = title(pax,...
    ['{\it ka} = ' ka_string]);
title1.FontSize = 14;

% title1 = suptitle(['Far-field normalized acoustic pressure for different values of {\it ka}']);
% title1.FontWeight = 'b';
% title1.FontSize = 18;

end

% title1 = suptitle('Normalized far-field scattered pressure for different values of {\itka} (hard sphere)');
% title1.FontWeight = 'b';
% title1.FontSize = 22;


% var_plot = 3;
% 
% figure();
% 
% ka_string = num2str(ka(var_plot));
% 
% %%%
% polarplot(phi,abs(n_ff_scattered_pressure(var_plot,:)),'-b','LineWidth',1);
% 
% pax = gca;
% pax.ThetaAxisUnits = 'radians';
% 
% rlim([0 0.5]);
% pax.RTick = [0 0.1 0.2 0.3 0.4 0.5];
% 
% % xlabel('x','Fontsize',10,'Fontweight','b');
% % ylabel('y','Fontsize',10,'Fontweight','b');
% 
% title1 = title(pax,...
%     ['{\bf Normalised far-field scattered pressure, }|{\it D}({\it 0},\phi{\bf ) / (}{\it aP_{0}}{\bf )| , for} {\it ka}{\bf = }' ka_string]);
% title1.FontSize = 14;


% 
% var_plot = 4;
% 
% figure();
% 
% ka_string = num2str(ka(var_plot));
% 
% %%%
% polarplot(phi,abs(n_ff_scattered_pressure(var_plot,:)),'-b','LineWidth',1);
% 
% pax = gca;
% pax.ThetaAxisUnits = 'radians';
% 
% rlim([0 1.2]);
% pax.RTick = [0 0.2 0.4 0.6 0.8 1 1.2];
% 
% % xlabel('x','Fontsize',10,'Fontweight','b');
% % ylabel('y','Fontsize',10,'Fontweight','b');
% 
% title1 = title(pax,...
%     ['{\bf Normalised far-field scattered pressure, }|{\it D}({\it 0},\phi{\bf ) / (}{\it aP_{0}}{\bf )| , for} {\it ka}{\bf = }' ka_string]);
% title1.FontSize = 14;

% 
% var_plot = 5;
% 
% figure();
% 
% ka_string = num2str(ka(var_plot));
% 
% %%%
% polarplot(phi,abs(n_ff_scattered_pressure(var_plot,:)),'-b','LineWidth',1);
% 
% pax = gca;
% pax.ThetaAxisUnits = 'radians';
% 
% rlim([0 2.5]);
% pax.RTick = [0 0.5 1 1.5 2 2.5];
% 
% % xlabel('x','Fontsize',10,'Fontweight','b');
% % ylabel('y','Fontsize',10,'Fontweight','b');
% 
% title1 = title(pax,...
%     ['{\bf Normalised far-field scattered pressure, }|{\it D}({\it 0},\phi{\bf ) / (}{\it aP_{0}}{\bf )| , for} {\it ka}{\bf = }' ka_string]);
% title1.FontSize = 14;


% var_plot = 6;
% 
% figure();
% 
% ka_string = num2str(ka(var_plot));
% 
% %%%
% polarplot(phi,abs(n_ff_scattered_pressure(var_plot,:)),'-b','LineWidth',1);
% 
% pax = gca;
% pax.ThetaAxisUnits = 'radians';
% 
% rlim([0 4.5]);
% pax.RTick = [0 1 2 3 4];
% 
% % xlabel('x','Fontsize',10,'Fontweight','b');
% % ylabel('y','Fontsize',10,'Fontweight','b');
% 
% title1 = title(pax,...
%     ['{\bf Normalised far-field scattered pressure, }|{\it D}({\it 0},\phi{\bf ) / (}{\it aP_{0}}{\bf )| , for} {\it ka}{\bf = }' ka_string]);
% title1.FontSize = 14;

% 
% var_plot = 7;
% 
% figure();
% 
% ka_string = num2str(ka(var_plot));
% 
% %%%
% polarplot(phi,abs(n_ff_scattered_pressure(var_plot,:)),'-b','LineWidth',1);
% 
% pax = gca;
% pax.ThetaAxisUnits = 'radians';
% 
% rlim([0 5]);
% pax.RTick = [0 1 2 3 4 5];
% 
% % xlabel('x','Fontsize',10,'Fontweight','b');
% % ylabel('y','Fontsize',10,'Fontweight','b');
% 
% title1 = title(pax,...
%     ['{\bf Normalised far-field scattered pressure, }|{\it D}({\it 0},\phi{\bf ) / (}{\it aP_{0}}{\bf )| , for} {\it ka}{\bf = }' ka_string]);
% title1.FontSize = 14;

